//+------------------------------------------------------------------+
//|  9:21 PM 4/30/2014                               polymorphic.mqh |
//|                               Copyright  2014, William H Roeder |
//|                                        mailto:WHRoeder@yahoo_com |
//| Provides generic class (Java equalivent of object) and the macro |
//| POLYMORPHIC (C++ equalivent <type> *p_subclass =                 |
//| dynamic_cast<<type> *>(p_obj) or Java's instanceof key word.)    |
//| Example of use:                                                  |
//|    class B : public cPolymorphic{                                |
//|     public: POLYMORPHIC(cPolymorphic);                           |
//|    };                                                            |
//|    class D : public B{                                           |
//|     public: POLYMORPHIC(B);                                      |
//|       :                                                          |
//|       B* b = new Other;                                          |
//|       D* d=(D*) b.dynamic_cast("D");  if(d) ...                  |
//|       if(b.IsA("D"){ D* d=(D*) b; ...                            |
//| See also http://forum.mql4.com/62793                             |
//+------------------------------------------------------------------+
#ifndef WHR_POLYMORPHIC_MQH
#define WHR_POLYMORPHIC_MQH
#include "utility.mqh"
class cPolymorphic{
 public:
   virtual             ~cPolymorphic(){}
   cPolymorphic*        dynamic_cast(string s)     const{
                                       return IsA(s)? GetPointer(this) : NULL; }
   virtual bool         IsA(const string& s)       const;
// #define POLYMORPHIC(BASE)                                                   \
//    virtual bool      IsA(const string& s)    const{return(s==typename(this) \
                                                            || BASE::IsA(s) ); }
#define POLYMORPHIC(B) virtual bool IsA(const string&s)const{return(s==typename(this)||B::IsA(s));}
}; // cPolymorphic 

// Mql4 doesn't have multiple inheritance.
class cPolymorphic_Noncopyable : public cPolymorphic{
 public:
   virtual             ~cPolymorphic_Noncopyable(){}
 private:               DISALLOW_COPY_AND_ASSIGN(cPolymorphic_Noncopyable);
};
//+- cPolymorphic ---------------------------------------------------+
bool         cPolymorphic::IsA(const string& s) const{return s==typename(this);}
//+------------------------------------------------------------------+
#endif   // WHR_POLYMORPHIC_MQH
